@@ -6,7 +6,7 @@ from distutils.spawn import spawn
 
 def pytest_configure(config):
     os.chdir(os.path.dirname(__file__))
-    cmd = [sys.executable, "setup.py", "-q", "build_ext", "-q"]
+    cmd = [sys.executable, "setup.py", "-q", "build_ext", "-q", "-i"]
     spawn(cmd, search_path=0)
 
     from tests import build_test_extensions
